package de.hftstuttgart.unifiedticketing.core;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.mockito.Mockito.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;


import static org.junit.jupiter.api.Assertions.*;

@ExtendWith(MockitoExtension.class)
public class TicketTest
{
    private Ticket instance;

    @BeforeEach
    public void initBeforeEach()
    {
        instance = mock(Ticket.class,
            withSettings().useConstructor(mock(TicketSystem.class)).defaultAnswer(CALLS_REAL_METHODS));
    }

    @Test
    public void testInitialValues()
    {
        TicketSystem parent = mock(TicketSystem.class);
        instance = mock(Ticket.class,
            withSettings().useConstructor(parent).defaultAnswer(CALLS_REAL_METHODS));

        assertAll(
            () -> assertNotNull(instance.assignees),
            () -> assertEquals(HashSet.class, instance.assignees.getClass()),
            () -> assertNull(instance.description),
            () -> assertNull(instance.description),
            () -> assertNull(instance.id),
            () -> assertNotNull(instance.labels),
            () -> assertEquals(HashSet.class, instance.labels.getClass()),
            () -> assertTrue(instance.open),
            () -> assertNull(instance.title),
            () -> assertNotNull(instance.updatedFields),
            () -> assertEquals(HashSet.class, instance.updatedFields.getClass()),
            () -> assertSame(parent, instance.parent)
            // constructor exception check with mock of abstract class not possible I guess?
//            () -> assertThrows(AssertionException.class,
//                () -> mock(Ticket.class, withSettings().useConstructor((TicketSystem) null)))
        );
    }

    @Test
    public void testGetAssignees()
    {
        assertEquals(instance.assignees, instance.getAssignees());
    }

    @Test
    public void testClearAssignees()
    {
        Set<TicketAssignee> expected = instance.assignees;
        instance.clearAssignees();
        assertSame(expected, instance.assignees);
    }

    @Test
    public void testGetDescription()
    {
        String expected = "some descriptive text";
        instance.description = expected;
        assertEquals(expected, instance.getDescription());
    }

    @Test
    public void testSetDescription()
    {
        String expected = "some other descriptive stuff";
        instance.setDescription(expected);
        assertEquals(expected, instance.description);
    }

    @Test
    public void testGetId()
    {
        String expected = "T13";
        instance.id = expected;
        assertEquals(expected, instance.getId());
    }

    @Test
    public void testAddLabel()
    {
        Set<String> expected = new HashSet<>(Arrays.asList("unifiedticketing", "bug", "feature"));
        expected.forEach(instance::addLabel);

        assertEquals(expected, instance.labels);
    }

    @Test
    public void testGetLabels()
    {
        Set<String> expected = new HashSet<>(Arrays.asList("unifiedticketing", "To Do", "enhancement"));
        instance.labels = expected;

        assertEquals(expected, instance.getLabels());
    }

    @Test
    public void testSetLabelsByList()
    {
        Set<String> expected = new HashSet<>(Arrays.asList("unifiedticketing", "Doing", "review"));
        instance.setLabels(expected);

        assertEquals(expected, instance.labels);
    }

    @Test
    public void testSetLabelsByArray()
    {
        String[] expected = new String[]{"unifiedticketing", "abandoned", "superseded"};
        instance.setLabels(expected);
        verify(instance, times(1)).setLabels(new HashSet<>(Arrays.asList(expected)));
    }

    @Test
    public void testIsOpen()
    {
        instance.open = true;
        assertTrue(instance.isOpen());

        instance.open = false;
        assertFalse(instance.isOpen());
    }

    @Test
    public void testOpen()
    {
        instance.open();
        assertTrue(instance.open);
    }

    @Test
    public void testClose()
    {
        instance.close();
        assertFalse(instance.open);
    }

    @Test
    public void testGetParent()
    {
        assertSame(instance.parent, instance.getParent());
    }

    @Test
    public void testGetTitle()
    {
        String expected = "fancy title";
        instance.title = expected;

        assertEquals(expected, instance.getTitle());
    }

    @Test
    public void testSetTitle()
    {
        String expected = "other title";
        instance.setTitle(expected);

        assertEquals(expected, instance.title);
    }
}
